@@ -32,15 +32,16 @@ If you need any help or assist, join our [discord](https://discord.gg/bDJCGD994p
 
 
 from .client import Components, Slash, UI
-from .components import ActionRow, Button, LinkButton, SelectMenu, SelectOption, Colors
+from .components import ActionRow, Button, LinkButton, SelectMenu, SelectOption, ButtonStyles
 from .slash.types import OptionType, SlashPermission, SlashOption
 from .slash.tools import ParseMethod
 from .tools import components_to_dict
-from .receive import Interaction, InteractionType, ResponseMessage, Message, WebhookMessage, PressedButton, SelectedMenu, SlashedCommand, SlashedSubCommand, EphemeralMessage, EphemeralResponseMessage
+from .slash.tools import create_choice
+from .receive import Interaction, InteractionType, Message, WebhookMessage, PressedButton, SelectedMenu, SlashedCommand, SlashedSubCommand, EphemeralMessage, EphemeralResponseMessage
 
 from .override import override_dpy, override_dpy2_client
 override_dpy2_client()
 
 
 __title__ = "discord-ui"
-__version__ = "3.3.5"
+__version__ = "4.0.0"
