@@ -1,11 +1,10 @@
-from discord.message import PartialMessage
-from discord_ui.slash.tools import handle_options
+from discord.ext.commands import Bot
 from .errors import InvalidEvent, OutOfValidRange, WrongType
 from .slash.errors import AlreadyDeferred, EphemeralDeletion
-from .slash.types import ContextCommand, SlashCommand, SubSlashCommand
+from .slash.types import ContextCommand, SlashCommand, SlashPermission, SlashSubcommand
 from .tools import MISSING, setup_logger
 from .http import BetterRoute, jsonifyMessage, send_files
-from .components import ActionRow, Button, ComponentType, LinkButton, SelectMenu, SelectOption, make_component
+from .components import ActionRow, Button, LinkButton, SelectMenu, SelectOption, make_component
 
 import discord
 from discord.ext.commands import Bot
@@ -18,26 +17,24 @@ logging = setup_logger("discord-ui")
 
 
 class InteractionType:
-    PING = 1
-    APPLICATION_COMMAND = 2
-    MESSAGE_COMPONENT = 3
+    PING                        =       Ping        =           1
+    APPLICATION_COMMAND         =      Command      =           2
+    MESSAGE_COMPONENT           =     Component     =           3
 
 class Interaction():
     def __init__(self, state, data, user=MISSING, message=None) -> None:
         self._state: ConnectionState = state
 
         self.deferred: bool = False
+        self.responded: bool = False
         self._deferred_hidden: bool = False
-        self._responded: bool = False
+        self._original_payload: dict = data
 
         if user is not MISSING:
-            self.member: typing.Union[discord.Member, discord.User] = user
-            """The user who created the interaction
-            
-            :type: :class:`discord.Member` | :class:`discord.User`
-            """
-        self._original_payload: dict = data
+            self.author: typing.Union[discord.Member, discord.User] = user
+            """The user who created the interaction"""
         self.application_id: int = data["application_id"]
+        """The ID of the bot application"""
         self.token: str = data["token"]
         """The token for responding to the interaction"""
         self.id: int = data["id"]
@@ -153,7 +150,7 @@ class Interaction():
                 else:
                     raise x
 
-        if self._responded is True:
+        if self.responded is True:
             return await self.send(content=content, tts=tts, embed=embed, embeds=embeds, nonce=nonce, allowed_mentions=allowed_mentions, mention_author=mention_author, components=components, hidden=hidden)
 
         
@@ -164,7 +161,7 @@ class Interaction():
                 logging.warning("Your response should be hidden, but the interaction was deferred public. This results in a public response.")
             if self._deferred_hidden is True and hidden is False:
                 logging.warning("Your response should be public, but the interaction was deferred hidden. This results in a hidden response.")
-        hide_message = self._deferred_hidden or not self.deferred and hidden
+        hide_message = self._deferred_hidden or not self.deferred and hidden is True
 
 
         r = None
@@ -180,9 +177,9 @@ class Interaction():
         if self.deferred is False and hide_message is False:
             route = BetterRoute("POST", f'/interactions/{self.id}/{self.token}/callback')
             r = await self._state.http.request(route, json={
-                    "type": 4,
-                    "data": payload
-                })
+                "type": 4,
+                "data": payload
+            })
         else:
             if self.deferred is False and hide_message is True:
                 await self.defer(hide_message)
@@ -191,7 +188,7 @@ class Interaction():
                 r = await send_files(route=route, files=[file] if files is MISSING else files, payload=payload, http=self._state.http)
             else:
                 r = await self._state.http.request(route, json=payload)
-        self._responded = True
+        self.responded = True
         
         if hide_message is True:
             return EphemeralMessage(state=self._state, channel=self._state.get_channel(int(r["channel_id"])), data=r, application_id=self.application_id, token=self.token)
@@ -199,7 +196,7 @@ class Interaction():
         
         if not hide_message:
             responseMSG = await self._state.http.request(BetterRoute("GET", f"/webhooks/{self.application_id}/{self.token}/messages/@original"))
-            msg = await getResponseMessage(self._state, data=responseMSG, response=False)
+            msg = await getMessage(self._state, data=responseMSG, response=False)
             if delete_after is not MISSING:
                 await msg.delete(delete_after)
             return msg
@@ -243,7 +240,7 @@ class Interaction():
             .. note::
                 If the response is hidden, a EphemeralMessage will be returned, which is an empty class
         """
-        if self._responded is False:
+        if self.responded is False:
             return await self.respond(content=content, tts=tts, embed=embed, embeds=embeds, file=file, files=files, nonce=nonce, allowed_mentions=allowed_mentions, mention_author=mention_author, components=components, hidden=hidden)
 
         payload = jsonifyMessage(content=content, tts=tts, embed=embed, embeds=embeds, nonce=nonce, allowed_mentions=allowed_mentions, mention_author=mention_author, components=components)
@@ -254,14 +251,13 @@ class Interaction():
         route = BetterRoute("POST", f'/webhooks/{self.application_id}/{self.token}')
         
         if file is not MISSING or files is not MISSING:
-            r = await send_files(route=route,
-                files=[file] if files is MISSING else files, payload=payload, http=self._state.http)
+            r = await send_files(route=route, files=[file] if files is MISSING else files, payload=payload, http=self._state.http)
         else:
             r = await self._state.http.request(route, json=payload)
 
         if hidden is True:
             return EphemeralMessage(state=self._state, channel=self._state.get_channel(r["channel_id"]), data=r, application_id=self.application_id, token=self.token)
-        return await getResponseMessage(self._state, r, response=False)
+        return await getMessage(self._state, r, response=False)
 
     def _handle_auto_defer(self, auto_defer):
         self.deferred = auto_defer[0]
@@ -269,70 +265,69 @@ class Interaction():
 
 class SelectedMenu(Interaction, SelectMenu):
     """A :class:`~SelectMenu` object in which an item was selected"""
-    def __init__(self, data, user, s, state, msg) -> None:
-        Interaction.__init__(self, state, data, user, msg)
+    def __init__(self, data, user, s, msg, client) -> None:
+        Interaction.__init__(self, client._connection, data, user, msg)
         SelectMenu.__init__(self, "EMPTY", [SelectOption("EMPTY", "EMPTY")], 0, 0)
         self._json = s.to_dict()
+        self.bot: Bot = client
+        #region selected_values
         self.selected_values: typing.List[SelectOption] = []
-        """The list of values which were selected
-        
-        :type: :class:`~SelectOption`
-        """
-        
+        """The list of values which were selected"""
         for val in data["data"]["values"]:
             for x in self.options:
                 if x.value == val:
                     self.selected_values.append(x)
-
-        self.member: discord.Member = user
-        """The member who selected the value"""
+        #endregion
+        self.author: discord.Member = user
+        """The user who selected the value"""
 
 class PressedButton(Interaction, Button):
     """A :class:`~Button` object that was pressed"""
-    def __init__(self, data, user, b, state, message) -> None:
-        Interaction.__init__(self, state, data, user, message)
+    def __init__(self, data, user, b, message, client) -> None:
+        Interaction.__init__(self, client._connection, data, user, message)
         Button.__init__(self, "empty", "empty")
         self._json = b.to_dict()
-
-        """interaction: :class:`dict`
-        
-        The most important stuff from the received interaction
-        
-        *  ``token``
-                The interaction token
-        *   ``id``
-                The ID for the interaction
-        """
-        self.member: discord.Member = user
+        self.bot: Bot = client
+        self.author: discord.Member = user
         """The user who pressed the button"""
 
 class SlashedCommand(Interaction, SlashCommand):
     """A :class:`~SlashCommand` object that was used"""
-    def __init__(self, client, command: SlashCommand, data, user, guild_ids = None) -> None:
+    def __init__(self, client, command: SlashCommand, data, user, args = None, guild_ids = None, guild_permissions = None) -> None:
         Interaction.__init__(self, client._connection, data, user)
-        SlashCommand.__init__(self, None, "EMPTY", guild_ids=guild_ids)
+        SlashCommand.__init__(self, None, "EMPTY", guild_ids=guild_ids, guild_permissions=guild_permissions)
+        self.bot: Bot = client
         self._json = command.to_dict()
-        self.member: discord.Member = user
+        self.author: discord.Member = user
         """The channel where the slash command was used"""
         self.guild_ids = guild_ids
-
-class SlashedSubCommand(SlashedCommand, SubSlashCommand):
+        """The ids of the guilds where the command is available"""
+        self.args: typing.Dict[str, typing.Union[str, int, bool, discord.Member, discord.TextChannel, discord.Role, float]] = args
+        """The options that were received"""
+        self.permissions: SlashPermission = guild_permissions.get(self.guild_id)
+        """The permissions for the guild"""
+class SlashedSubCommand(SlashedCommand, SlashSubcommand):
     """A Sub-:class:`~SlashCommand` command that was used"""
-    def __init__(self, client, command, data, user, guild_ids) -> None:
-        SlashedCommand.__init__(self, client, command, data, user, guild_ids=guild_ids)
-        SubSlashCommand.__init__(self, None, "EMPTY", "EMPTY")
-
+    def __init__(self, client, command, data, user, args = None, guild_ids = None, guild_permissions=None) -> None:
+        SlashedCommand.__init__(self, client, command, data, user, args, guild_ids=guild_ids, guild_permissions=guild_permissions)
+        SlashSubcommand.__init__(self, None, "EMPTY", "EMPTY")
 
 class SlashedContext(Interaction, ContextCommand):
-    def __init__(self, client, command: ContextCommand, data, user, guild_ids = None) -> None:
+    def __init__(self, client, command: ContextCommand, data, user, param, guild_ids = None, guild_permissions = None) -> None:
         Interaction.__init__(self, client._connection, data, user)
-        ContextCommand.__init__(self, None, "EMPTY", guild_ids)
+        ContextCommand.__init__(self, None, "EMPTY", guild_ids=guild_ids, guild_permissions=guild_permissions)
+        self.bot: Bot = client
         self._json = command.to_dict()
-        self.member: discord.Member = user
-        self.guild_ids = guild_ids
+        self.guild_ids: typing.List[int] = guild_ids
+        """The guild_ids where the command is available"""
+        self.param: typing.Union[Message, discord.Member, discord.User] = param
+        """The parameter that was received"""
+        self.permissions: SlashPermission = guild_permissions.get(self.guild_id)
+        """The permissions for the guild"""
+        
 
 
-async def getResponseMessage(state: ConnectionState, data, user=None, response = True):
+async def getMessage(state: ConnectionState, data, response = True):
     """
     Async function to get the response message
 
@@ -350,21 +345,18 @@ async def getResponseMessage(state: ConnectionState, data, user=None, response =
 
     Returns
     -------
-    :class:`~Message` | :class:`~ResponseMessage`
+    :class:`~Message` | :class:`~EphemeralMessage`
         The sent message
-
-    .. note::
-            If the message comes from an interaction, it will be of type :class:`~ResponseMessage`, if it is sent to a textchannel, it will be of type :class:`~Message`
     """
     channel = state.get_channel(int(data["channel_id"]))
-    if response and user:
-        if data.get("message") is not None and data["message"]["flags"] == 64:
-            return EphemeralResponseMessage(state=state, channel=channel, data=data, user=user)
-        return ResponseMessage(state=state, channel=channel, data=data, user=user)
+    if response:
+        if data.get("message") is not None and data.get("message", data)["flags"] == 64:
+            return EphemeralResponseMessage(state=state, channel=channel, data=data.get("message", data))
+        return Message(state=state, channel=channel, data=data.get("message", data))
 
     if data.get("message") is not None and data["message"]["flags"] == 64:
-        return EphemeralMessage(state=state, channel=channel, data=data["message"])
-    return Message(state=state, channel=channel, data=data)
+        return EphemeralMessage(state=state, channel=channel, data=data.get("message", data))
+    return Message(state=state, channel=channel, data=data.get("message", data))
 
 class Message(discord.Message):
     """A fixed :class:`discord.Message` optimized for components"""
@@ -385,7 +377,7 @@ class Message(discord.Message):
 
     # region attributes
     @property
-    def buttons(self):
+    def buttons(self) -> typing.List[typing.Union[Button, LinkButton]]:
         """The button components in the message
         
         :type: List[:class:`~Button` | :class:`~LinkButton`]
@@ -394,7 +386,7 @@ class Message(discord.Message):
             return [x for x in self.components if type(x) in [Button, LinkButton]]
         return []
     @property
-    def select_menus(self):
+    def select_menus(self) -> typing.List[SelectMenu]:
         """The select menus components in the message
 
         :type: List[:class:`~SelectMenu`]
@@ -536,35 +528,26 @@ class Message(discord.Message):
             rows.append(c_row) 
         return rows
 
-    @typing.overload
-    async def wait_for(self, client, event_name="button") -> PressedButton: ...
-    @typing.overload 
-    async def wait_for(self, client, event_name="select") -> SelectedMenu: ...
-    @typing.overload
-    async def wait_for(self, client, event_name, custom_id) -> PressedButton: ...
-    @typing.overload
-    async def wait_for(self, client, event_name, custom_id) -> SelectedMenu: ...
-    @typing.overload
-    async def wait_for(self, client, event_name, timeout) -> PressedButton: ...
-    @typing.overload
-    async def wait_for(self, client, event_name, timeout) -> SelectedMenu: ...
-    async def wait_for(self, client, event_name: typing.Literal["select", "button"], custom_id=MISSING, timeout=None) -> typing.Union[PressedButton, SelectedMenu]:
+    async def wait_for(self, event_name: typing.Literal["select", "button"], client, custom_id=MISSING, by=None, check=lambda component: True, timeout=None) -> typing.Union[PressedButton, SelectedMenu]:
         """Waits for a message component to be invoked in this message
 
         Parameters
         -----------
-        client: :class:`discord.ext.commands.Bot`
-            The discord client
         event_name: :class:`str`
             The name of the event which will be awaited [``"select"`` | ``"button"``] 
-
             .. note::
 
                 The event_name must be ``select`` for a select menu selection and ``button`` for a button press
-        
+
+        client: :class:`discord.ext.commands.Bot`
+            The discord client
         custom_id: :class:`str`, Optional
             Filters the waiting for a custom_id
-        
+        by: :class:`discord.User` | :class:`discord.Member` | :class:`int` | :class:`str`, Optional
+            The user or the user id by that has to create the component interaction
+        check: :class:`function`, Optional
+            A check that has to return True in order to break from the event and return the received component
+                The function takes the received component as the parameter
         timeout: :class:`float`, Optional
             After how many seconds the waiting should be canceled. 
             Throws an :class:`asyncio.TimeoutError` Exception
@@ -579,34 +562,30 @@ class Message(discord.Message):
         :type: :class:`~PressedButton` | :class:`~SelectedMenu`
         """
         if event_name.lower() in ["button", "select"]:
-            def check(btn, msg):
+            def _check(com, msg):
                 if msg.id == self.id:
-                    if custom_id is not MISSING and btn.custom_id == custom_id:
-                        return True
-                    return True
+                    statements = []
+                    if custom_id is not MISSING:
+                        statements.append(com.custom_id == custom_id)
+                    if by is not MISSING:
+                        statements.append(com.member.id == (by.id if hasattr(by, "id") else int(by)))
+                    if check is not MISSING:
+                        statements.append(check(com))
+                    return all(statements)
+                return False
             if not isinstance(client, Bot):
                 raise WrongType("client", client, "discord.ext.commands.Bot")
-            return (await client.wait_for('button_press' if event_name.lower() == "button" else "menu_select", check=check, timeout=timeout))[0]
+            return (await client.wait_for('button_press' if event_name.lower() == "button" else "menu_select", check=_check, timeout=timeout))
         
         raise InvalidEvent(event_name, ["button", "select"])
 
-
 class ResponseMessage(Message):
     r"""A message Object which extends the `Message` Object optimized for an interaction component"""
-    def __init__(self, *, state, channel, data, user):
+    def __init__(self, *, state, channel, data, user, interaction_component = None):
         Message.__init__(self, state=state, channel=channel, data=data["message"])
         self.interaction = Interaction(state, data, user, self)
+        self.interaction_component = interaction_component
 
-        self.interaction_component = None
-
-        if int(data["data"]["component_type"]) == 2:
-            for x in self.buttons:
-                if hasattr(x, 'custom_id') and x.custom_id == data["data"]["custom_id"]:
-                    self.interaction_component = PressedButton(data, user, x, self._state, self)
-        elif int(data["data"]["component_type"]) == 3:
-            for x in self.select_menus:
-                if x.custom_id == data["data"]["custom_id"]:
-                    self.interaction_component = SelectedMenu(data, user, x, self._state, self)
 
 class WebhookMessage(Message, discord.WebhookMessage):
     def __init__(self, *, state, channel, data):
@@ -662,15 +641,15 @@ class EphemeralMessage(Message):
     async def delete(self):
         raise EphemeralDeletion()
 
-class EphemeralResponseMessage(ResponseMessage):
+class EphemeralResponseMessage(Message):
     """A ephemeral message wich was created from an interaction
     
     .. important::
 
         Methods like `.edit()`, which change the original message, need a `token` paremeter passed in order to work
     """
-    def __init__(self, *, state, channel, data, user):
-        ResponseMessage.__init__(self, state=state, channel=channel, data=data, user=user)
+    def __init__(self, *, state, channel, data):
+        Message.__init__(self, state=state, channel=channel, data=data)
 
     async def edit(self, token, **fields):
         """Edits the message
@@ -688,7 +667,7 @@ class EphemeralResponseMessage(ResponseMessage):
 
                 async def testing(ctx):
                     msg = await ctx.send("hello hidden world", components=[Button("test")])
-                    btn = await msg.wait_for(client, "button")
+                    btn = await msg.wait_for("button", client)
                     await btn.message.edit(ctx.token, content="edited", components=None)
         
         """
